@@ -8,7 +8,7 @@ import alpaca_trade_api as tradeapi
 #           AlgoLogger
 #           Backtester
 #           Datasource
-#
+#           Stream
 #
 
 class AlgoLogger:
@@ -20,28 +20,35 @@ class AlgoLogger:
         self.warn_print =  " [  -------- WARN  --------  ] "
         self.sep =         " [  -----------------------  ] "
         self.fn = data_path + "daily_logs/prog/{}.txt".format(datetime.today().strftime('%Y-%m-%d'))
-    
+
     def info(self, output):
         self.to_print.append(self.info_print + str(output))
+        self.output()
     def trade(self, output):
         self.to_print.append(self.trade_print + str(output))
+        self.output()
     def warn(self, output):
         self.to_print.append(self.warn_print + str(output))
+        self.output()
     def error(self, output):
         self.to_print.append(self.error_print + str(output))
-    
+        self.output()
+
     def set_name(self, name):
         self.error_print = " [ " + name + " -------- ERROR --------  ] "
         self.trade_print = " [ " + name + " -------- TRADE --------  ] "
         self.info_print =  " [ " + name + " -------- INFO  --------  ] "
         self.warn_print =  " [ " + name + " -------- WARN  --------  ] "
-    
+
     def output(self):
-        log_file = open(self.fn, 'a')
         for p in self.to_print:
             print(p)
-            log_file.write(p)
-        log_file.close()
+            try:
+                log_file = open(self.fn, 'a')
+                log_file.write(p)
+                log_file.close()
+            except:
+                pass
         self.clear_print_buf()
     def output_scr(self):
         i=0
@@ -52,7 +59,7 @@ class AlgoLogger:
         print(self.sep * 4)
     def clear_print_buf(self):
         self.to_print = []
-    
+
 log = AlgoLogger()
 
 backtest = "backtest"
@@ -81,7 +88,7 @@ class Backtester():
             self.positions[t] = 0
         self.update_dataframes()
         self.run()
-    
+
     # dictates if algo is done overall
     def run(self, tick_period):
         if self.cur_time >= self.end_utc or self.algo_done:
@@ -90,7 +97,7 @@ class Backtester():
             return
         else:
             self.run_step()
-    
+
     # dictates time between market close and opens
     def run_step(self):
         minute = 60
@@ -102,7 +109,7 @@ class Backtester():
                 self.algo_done = True
                 self.is_open = False
                 return
-        
+
         # now check that the current data frame is valid for each ticker
         # if the dataframe isn't valid, try to get a new dataframe
         for t in self.tickers:
@@ -115,7 +122,7 @@ class Backtester():
                     self.cur_time += (minute * 60 * 23)
                     self.open_hours()
                     self.update_dataframes()
-                    
+
                     if self.cur_time >= self.end_utc or self.algo_done:
                         self.algo_done = True
                         self.is_open = False
@@ -124,7 +131,7 @@ class Backtester():
     def update_dataframes(self):
         for t in self.tickers:
             self.cur_df[t] = self.poly.historic_agg_v2(t, 1, 'minute', _from=self.current_date, to=self.current_date).df
-    
+
     def open_hours(self):
         self.cur_utc = datetime.fromtimestamp(self.cur_time, timezone(-timedelta(hours=4)))
         self.current_date = "{0:0=4d}".format(self.cur_utc.year)+"-"+"{0:0=2d}".format(self.cur_utc.month)+"-"+"{0:0=2d}".format(self.cur_utc.day)
@@ -134,73 +141,67 @@ class Backtester():
             return True
         else:
             return False
-            
+
     def get_clock(self):
         return {"is_open" : self.is_open}
-        
+
     def get_last_trade(self, ticker):
         price = self.cur_df[ticker].loc[self.cur_utc]
         ret = {"price" : price["close"]}
         return ret
-        
+
     def get_position(self, ticker):
         ret = {"qty" : self.positions[ticker]}
         return ret
-        
+
     def get_account(self):
         ret = {"cash" : self.account}
         return ret
-    
+
     def submit_order(self, ticker, quantity, side, type, limit_price, time_in_force):
         # check using cur_price vs limit_price
         # cur_price = get_last_trade(ticker)
         if side == "buy":
             self.positions[ticker] += quantity
             self.account -= limit_price * quantity
-            
+
         if side == "sell":
             self.positions[ticker] -= quantity
             self.account += limit_price * quantity
         return
 
 class DataSource:
-    
+
     def __init__(self, key_id, secret_key, base_url, data_source=backtest, tickers=None, start_date=None, end_date=None):
-        
         self.api_endpoint = tradeapi.REST(
             key_id=key_id,
             secret_key=secret_key,
             base_url=base_url
         )
-        
         self.mode = data_source      # backtest, live, paper
-        self.test_mode = True
+        self.quick_test = True
         if self.mode == backtest:
             self.backtester = Backtester(self.api_endpoint, tickers, start_date, end_date)
         else:
             self.backtester = None
-            
 
-    # 
+
+    #
     # tick_perod is in seconds
     #
     def step(self, tick_period):
         if (self.mode == live or self.mode == paper):
-                    if self.test_mode:
+                    if self.quick_test:
                         time.sleep(0.5)
                     else:
                         time.sleep(tick_period)
         if (self.mode == backtest):
             self.backtester.run(tick_period)
-            
-            
+
+
     def get_clock(self):
         if (self.mode == backtest):
             return self.backtester.get_clock()
-        # if (self.mode == paper):
-        #     clk = api_paper.get_clock()
-        # if (self.mode == live):
-        #     clk = api_live.get_clock()
         clk = self.api_endpoint.get_clock()
         ret = {"is_open" : clk.is_open}
         return ret
@@ -208,10 +209,6 @@ class DataSource:
     def get_last_trade(self, ticker):
         if (self.mode == backtest):
             return self.backtester.get_last_trade(ticker)
-        # if (self.mode == paper):
-        #     trd = api_paper.get_last_trade(ticker)
-        # if (self.mode == live):
-        #     trd = api_live.get_last_trade(ticker)
         trd = self.api_endpoint.get_last_trade(ticker)
         ret = {"price" : trd.price}
         return ret
@@ -219,29 +216,21 @@ class DataSource:
     def get_position(self, ticker):
         if (self.mode == backtest):
             return self.backtester.get_position(ticker)
-        # if (self.mode == paper):
-        #     pos = api_paper.get_position(ticker)
-        # if (self.mode == live):
-        #     pos = api_live.get_position(ticker)
         try:
             qty =  self.api_endpoint.get_position(ticker).qty
         except:
             qty = 0
         ret = {"qty" : qty}
         return ret
-        
+
     def get_account(self):
         if (self.mode == backtest):
             return self.backtester.get_account()
-        # if (self.mode == paper):
-        #     act = api_paper.get_account()
-        # if (self.mode == live):
-        #     act = api_live.get_account()
         act = self.api_endpoint.get_account()
         ret = {"cash" : act.cash}
         return ret
 
-    def list_orders(self, ticker):
+    def list_orders(self, side):
         try:
             if (self.mode == backtest):
                 return self.backtester.list_orders()
@@ -251,15 +240,12 @@ class DataSource:
                 orders = api_live.list_orders()
             valid_orders = []
             for o in orders:
-                log.info(o)
-                if o.side == "sell" and o.symbol == ticker:
-                    valid_orders.append(o)
-            log.info(valid_orders[0])
-            log.info("---------")
-            log.info(valid_orders[len(valid_orders)-1])
-            # log.info(orders)
-            ret = {"cash" : act.cash}
-            return ret
+                if o.side == side:
+                    info = {"limit_price" : o.limit_price,
+                            "qty" : o.qty,
+                            "id" : o.id}
+                    valid_orders.append(info)
+            return valid_orders
         except Exception as e:
             print(e)
             print("list_orders : IMPLEMENT THIS")
@@ -269,15 +255,63 @@ class DataSource:
         try:
             if (self.mode == backtest):
                 return self.backtester.submit_order(ticker, quantity, side, type, limit_price, time_in_force)
-            self.api_endpoint.submit_order(
-                symbol=ticker,
-                qty=quantity,
-                side=side,
-                type=type,
-                limit_price=limit_price,
-                time_in_force=time_in_force
-                )
+            response = self.api_endpoint.submit_order(
+            symbol=ticker,
+            qty=quantity,
+            side=side,
+            type=type,
+            limit_price=limit_price,
+            time_in_force=time_in_force
+            )
+            info = {"limit_price" : response.limit_price,
+                    "qty" : response.qty,
+                    "id" : response.id}
+            return info
 
         except Exception as e:
             log.info("DATA SOURCE ERROR: submit_order()         " + str(e))
+    def cancel_order(self, order_id):
+        try:
+            if (self.mode == backtest):
+                return
+            self.api_endpoint.cancel_order(
+            order_id=order_id
+            )
+        except Exception as e:
+            log.info("CANCEL ORDER ERROR: cancel_order()        " + str(e))
 
+class Stream:
+
+    def __init__(self, key_id, secret_key, base_url):
+        self.open_orders = {}
+        try:
+            self.conn  = tradeapi.stream2.StreamConn(key_id, secret_key, base_url)
+        except Exception as e:
+            log.error("CANNOT ESTABLISH WEB SOCKET CONNECTION:          " + str(e))
+        @self.conn.on(r'^trade_updates$')
+        async def on_trade_updates(conn, channel, trade):
+            if trade.event == 'new':
+                side = trade.order['side']
+                symbol = trade.order['symbol']
+                id = trade.order['id']
+                qty = trade.order['qty']
+                limit_price = trade.order['limit_price']
+                self.open_orders[id] = (side, qty, limit_price, symbol)
+                print(len(self.open_orders))
+            elif trade.event == 'canceled':
+                del self.open_orders[trade.order['id']]
+                print(len(self.open_orders))
+            elif trade.event == 'fill':
+                del self.open_orders[trade.order['id']]
+                print(len(self.open_orders))
+            elif trade.event == 'expired':
+                del self.open_orders[trade.order['id']]
+                print(len(self.open_orders))
+
+
+    def ws_start(self):
+        self.conn.run(['trade_updates'])
+        time.sleep(2)
+
+    def get_open_orders(self ):
+        return self.open_orders
